首页
学习
活动
专区
圈层
工具
发布
社区首页 >问答首页 >异常后重置Cuda上下文

异常后重置Cuda上下文
EN

Stack Overflow用户
提问于 2019-05-27 16:08:49
回答 1查看 2.9K关注 0票数 3

我有一个使用Cuda / C++的工作应用程序,但有时由于内存泄漏,会抛出异常。我需要能够重置GPU在现场,我的应用程序是一个服务器,所以它必须保持可用。

我试过这样的方法,但似乎行不通:

代码语言:javascript
复制
try
{
    // do process using GPU
}
catch (std::exception &e)
{
    // catching exception from cuda only

    cudaSetDevice(0);
    CUDA_RETURN_(cudaDeviceReset());
}

我的想法是每次我从GPU获得一个异常时重新设置设备,但我无法使它正常工作。:(顺便说一下,出于某些原因,我不能解决我的Cuda代码中的每一个问题,我需要一个临时的解决方案。谢谢!

EN

回答 1

Stack Overflow用户

发布于 2019-05-27 17:51:45

在不可恢复(“粘滞”) CUDA错误之后恢复适当设备功能的唯一方法是终止发起(即发出导致错误的CUDA运行时API调用)的主机进程

因此,对于单个进程应用程序,唯一的方法是终止应用程序。

应该可以设计一个多进程应用程序,其中初始(“父”)进程不使用任何CUDA,并生成一个使用GPU的子进程。当子进程遇到不可恢复的CUDA错误时,它必须终止。

父进程可以选择性地监视子进程。如果它确定子进程已经终止,它可以重新生成该进程并恢复CUDA功能行为。

粘性和非粘性错误都在其他地方,比如here

一个正确的多进程应用程序的例子,它使用例如fork()来生成一个使用CUDA的子进程,这个示例可以在CUDA示例代码simpleIPC中找到。下面是从simpleIPC示例(用于linux)中组装的一个粗略示例:

代码语言:javascript
复制
$ cat t477.cu
/*
 * Copyright 1993-2015 NVIDIA Corporation.  All rights reserved.
 *
 * Please refer to the NVIDIA end user license agreement (EULA) associated
 * with this source code for terms and conditions that govern your use of
 * this software. Any use, reproduction, disclosure, or distribution of
 * this software and related documentation outside the terms of the EULA
 * is strictly prohibited.
 *
 */

// Includes
#include <stdio.h>
#include <assert.h>

// CUDA runtime includes
#include <cuda_runtime_api.h>

// CUDA utilities and system includes
#include <helper_cuda.h>

#define MAX_DEVICES          1
#define PROCESSES_PER_DEVICE 1
#define DATA_BUF_SIZE        4096

#ifdef __linux
#include <unistd.h>
#include <sched.h>
#include <sys/mman.h>
#include <sys/wait.h>
#include <linux/version.h>

typedef struct ipcDevices_st
{
    int count;
    int results[MAX_DEVICES];
} ipcDevices_t;


// CUDA Kernel
__global__ void simpleKernel(int *dst, int *src, int num)
{
    // Dummy kernel
    int idx = blockIdx.x * blockDim.x + threadIdx.x;
    dst[idx] = src[idx] / num;
}


void runTest(int index, ipcDevices_t* s_devices)
{
    if (s_devices->results[0] == 0){
        simpleKernel<<<1,1>>>(NULL, NULL, 1);  // make a fault
        cudaDeviceSynchronize();
        s_devices->results[0] = 1;}
    else {
        int *d, *s;
        int n = 1;
        cudaMalloc(&d, n*sizeof(int));
        cudaMalloc(&s, n*sizeof(int));
        simpleKernel<<<1,1>>>(d, s, n);
        cudaError_t err = cudaDeviceSynchronize();
        if (err != cudaSuccess)
          s_devices->results[0] = 0;
        else
          s_devices->results[0] = 2;}
    cudaDeviceReset();
}
#endif

int main(int argc, char **argv)
{

    ipcDevices_t *s_devices = (ipcDevices_t *) mmap(NULL, sizeof(*s_devices),
                                                    PROT_READ | PROT_WRITE, MAP_SHARED | MAP_ANONYMOUS, 0, 0);
    assert(MAP_FAILED != s_devices);

    // We can't initialize CUDA before fork() so we need to spawn a new process
    s_devices->count = 1;
    s_devices->results[0] = 0;

    printf("\nSpawning child process\n");
    int index = 0;

    pid_t pid = fork();

    printf("> Process %3d\n", pid);
    if (pid == 0) { // child process
    // launch our test
      runTest(index, s_devices);
    }
    // Cleanup and shutdown
    else { // parent process
            int status;
            waitpid(pid, &status, 0);
            if (s_devices->results[0] < 2) {
              printf("first process launch reported error: %d\n", s_devices->results[0]);
              printf("respawn\n");
              pid_t newpid = fork();
              if (newpid == 0) { // child process
                    // launch our test
                 runTest(index, s_devices);
                  }
    // Cleanup and shutdown
              else { // parent process
                int status;
                waitpid(newpid, &status, 0);
                if (s_devices->results[0] < 2)
                  printf("second process launch reported error: %d\n", s_devices->results[0]);
                else
                  printf("second process launch successful\n");
                }

            }

    }

    printf("\nShutting down...\n");

    exit(EXIT_SUCCESS);

}
$ nvcc -I/usr/local/cuda/samples/common/inc t477.cu -o t477
$ ./t477

Spawning child process
> Process 10841
> Process   0

Shutting down...
first process launch reported error: 1
respawn

Shutting down...
second process launch successful

Shutting down...
$

对于windows,唯一需要的更改应该是使用windows机制进行主机进程间通信。

票数 6
EN
页面原文内容由Stack Overflow提供。腾讯云小微IT领域专用引擎提供翻译支持
原文链接:

https://stackoverflow.com/questions/56329377

复制
相关文章

相似问题

领券
问题归档专栏文章快讯文章归档关键词归档开发者手册归档开发者手册 Section 归档